package org.jlab.coda.jevio.test;

import org.jlab.coda.jevio.*;

import java.io.IOException;
import java.nio.*;
import java.util.Arrays;
import java.util.List;
import java.util.Scanner;

/**
 * Take test programs out of EvioCompactReader.
 * @author timmer
 * Date: Nov 15, 2012
 */
public class CompactReaderTest {

    /**
     * This method prints out a portion of a given ByteBuffer object
     * in hex representation of ints.
     *
     * @param buf buffer to be printed out
     * @param lenInInts length of data in ints to be printed
     */
    private static void printBuffer(ByteBuffer buf, int lenInInts) {
        IntBuffer ibuf = buf.asIntBuffer();
        lenInInts = lenInInts > ibuf.capacity() ? ibuf.capacity() : lenInInts;
        for (int i=0; i < lenInInts; i++) {
            System.out.println("  Buf(" + i + ") = 0x" + Integer.toHexString(ibuf.get(i)));
        }
    }



    static EvioEvent createSingleEvent(int tag) {

        // Top level event
        EvioEvent event = null;

        // data  (1500 bytes)
        int[] intData = new int[2];
        Arrays.fill(intData, 1);

        byte[] byteData = new byte[9];
        Arrays.fill(byteData, (byte)2);

        short[] shortData = new short[3];
        Arrays.fill(shortData, (short)3);

        double[] doubleData = new double[1];
        Arrays.fill(doubleData, 4.);

        try {

            // Build event (bank of banks) with EventBuilder object
            EventBuilder builder = new EventBuilder(tag, DataType.BANK, 1);
            event = builder.getEvent();

                // bank of banks
                EvioBank bankBanks = new EvioBank(tag+1, DataType.BANK, 2);
                builder.addChild(event, bankBanks);

                    // bank of ints
                    EvioBank bankInts = new EvioBank(tag+2, DataType.INT32, 3);
                    bankInts.appendIntData(intData);
                    builder.addChild(bankBanks, bankInts);

                // Bank of segs
                EvioBank bankBanks2 = new EvioBank(tag+3, DataType.BANK, 4);
                builder.addChild(event, bankBanks2);

                    // segment of shorts
                    EvioBank bankShorts = new EvioBank(tag+4, DataType.SHORT16, 5);
                    bankShorts.appendShortData(shortData);
                    builder.addChild(bankBanks2, bankShorts);

                    // segment of segments
                    EvioBank bankBanks3 = new EvioBank(tag+5, DataType.BANK, 6);
                    builder.addChild(bankBanks2, bankBanks3);

                        // segment of doubles
                        EvioBank bankDoubles = new EvioBank(tag+6, DataType.DOUBLE64, 7);
                        bankDoubles.appendDoubleData(doubleData);
                        builder.addChild(bankBanks3, bankDoubles);

                // Bank of tag segs
                EvioBank bankBanks4 = new EvioBank(tag+7, DataType.BANK, 8);
                builder.addChild(event, bankBanks4);

                    // tag segment of bytes
                    EvioBank bankBytes = new EvioBank(tag+8, DataType.CHAR8, 9);
                    bankBytes.appendByteData(byteData);
                    builder.addChild(bankBanks4, bankBytes);


        }
        catch (EvioException e) {
            e.printStackTrace();
        }

        return event;

    }


    static EvioEvent createSimpleSingleEvent(int tag) {

        // Top level event
        EvioEvent event = null;

        // data
        int[] intData = new int[21];
        Arrays.fill(intData, 1);

        try {
            // Build event (bank of banks) with EventBuilder object
            EventBuilder builder = new EventBuilder(tag, DataType.BANK, 1);
            event = builder.getEvent();

            // bank of ints
            EvioBank bankInts = new EvioBank(tag+1, DataType.INT32, 2);
            bankInts.appendIntData(intData);
            builder.addChild(event, bankInts);

        }
        catch (EvioException e) {
            e.printStackTrace();
        }

        return event;

    }




    static EvioEvent createSingleEvent1(int tag) {

        // Top level event
        EvioEvent event = null;

        // data  (1500 bytes)
        int[] intData = new int[4];
        Arrays.fill(intData, 1);

        byte[] byteData = new byte[4];
        Arrays.fill(byteData, (byte)2);

        short[] shortData = new short[4];
        Arrays.fill(shortData, (short)3);

        double[] doubleData = new double[5];
        Arrays.fill(doubleData, 4.);

        try {

            // Build event (bank of banks) with EventBuilder object
            EventBuilder builder = new EventBuilder(tag, DataType.BANK, 1);
            event = builder.getEvent();

                // bank of banks
                EvioBank bankBanks = new EvioBank(tag+1, DataType.BANK, 2);
                builder.addChild(event, bankBanks);

                    // bank of ints
                    EvioBank bankInts = new EvioBank(tag+2, DataType.INT32, 3);
                    bankInts.appendIntData(intData);
                    builder.addChild(bankBanks, bankInts);

                // Bank of segs
                EvioBank bankSegs = new EvioBank(tag+3, DataType.SEGMENT, 1);
                builder.addChild(event, bankSegs);

                    // segment of shorts
                    EvioSegment segShorts = new EvioSegment(tag+4, DataType.SHORT16);
                    segShorts.appendShortData(shortData);
                    builder.addChild(bankSegs, segShorts);

                    // segment of segments
                    EvioSegment segSegs = new EvioSegment(tag+5, DataType.SEGMENT);
                    builder.addChild(bankSegs, segSegs);

                        // segment of doubles
                        EvioSegment segDoubles = new EvioSegment(tag+6, DataType.DOUBLE64);
                        segDoubles.appendDoubleData(doubleData);
                        builder.addChild(segSegs, segDoubles);

                // Bank of tag segs
                EvioBank bankTagsegs = new EvioBank(tag+7, DataType.TAGSEGMENT, 1);
                builder.addChild(event, bankTagsegs);

                    // tag segment of bytes
                    EvioTagSegment tagBytes = new EvioTagSegment(tag+8, DataType.CHAR8);
                    tagBytes.appendByteData(byteData);
                    builder.addChild(bankTagsegs, tagBytes);


        }
        catch (EvioException e) {
            e.printStackTrace();
        }

        return event;

    }

    static EvioEvent createSingleEvent2(int tag) {

        // Top level event
        EvioEvent event = null;

        // data  (1500 bytes)
        int[] intData = new int[1];
        Arrays.fill(intData, 1);

        byte[] byteData = new byte[4];
        Arrays.fill(byteData, (byte)2);

        short[] shortData = new short[2];
        Arrays.fill(shortData, (short)3);

        double[] doubleData = new double[1];
        Arrays.fill(doubleData, 4.);

        try {

            // Build event (bank of banks) with EventBuilder object
            EventBuilder builder = new EventBuilder(tag, DataType.SEGMENT, 1);
            event = builder.getEvent();


                    // segment of shorts
                    EvioSegment segShorts = new EvioSegment(tag+4, DataType.SHORT16);
                    segShorts.appendShortData(shortData);
                    builder.addChild(event, segShorts);


                    // segment of doubles
                    EvioSegment segDoubles = new EvioSegment(tag+6, DataType.DOUBLE64);
                    segDoubles.appendDoubleData(doubleData);
                    builder.addChild(event, segDoubles);


//                // Bank of tag segs
//                EvioBank bankTagsegs = new EvioBank(tag+7, DataType.TAGSEGMENT, 1);
//                builder.addChild(event, bankTagsegs);
//
//                    // tag segment of bytes
//                    EvioTagSegment tagBytes = new EvioTagSegment(tag+8, DataType.CHAR8);
//                    tagBytes.appendByteData(byteData);
//                    builder.addChild(bankTagsegs, tagBytes);


        }
        catch (EvioException e) {
            e.printStackTrace();
        }

        return event;

    }





    static ByteBuffer createSingleSegment(int tag) {

        EvioSegment seg = new EvioSegment(tag, DataType.INT32);

        int[] intData = new int[10];
        Arrays.fill(intData, 456);

        try {
            seg.appendIntData(intData);
        }
        catch (EvioException e) {
            e.printStackTrace();
        }

        ByteBuffer bb = ByteBuffer.allocate(4*12);
        seg.write(bb);
        bb.flip();

        return bb;
    }



    static ByteBuffer createSingleBank(int tag, int num) {

         EvioEvent bank = createSingleEvent1(tag);
         int byteSize = bank.getTotalBytes();

         ByteBuffer bb = ByteBuffer.allocate(byteSize);
         bank.write(bb);
         bb.flip();

         return bb;
     }

    static ByteBuffer createSingleBank2(int tag, int num) {

         EvioBank bank = new EvioBank(tag, DataType.INT32, num);

         int[] intData = new int[10];
         Arrays.fill(intData, 123);

         try {
             bank.appendIntData(intData);
         }
         catch (EvioException e) {
             e.printStackTrace();
         }

         ByteBuffer bb = ByteBuffer.allocate(4*12);
         bank.write(bb);
         bb.flip();

         return bb;
     }


    static ByteBuffer createBuffer(int eventCount) {

        String xmlDict = null;

        String xmlDict2 =
                "<xmlDict attr='junk'>"  +
                    "<dictEntry name='T7N0'    tag= '7'    num = '0' />"  +
                    "<dictEntry name='T3N3'    tag= '3'    num = '3' />"  +
                    "<dictEntry name='T66N99'  tag= '66'   num = '99' />"  +
                "</xmlDict>";

        // Create a buffer
        ByteBuffer myBuf = ByteBuffer.allocate(1000 * eventCount);
//        myBuf.position(100);

        try {
            // Create an event writer to write into "myBuf" with 10000 ints or 100 events/block max
            EventWriter writer = new EventWriter(myBuf, 10000, 100, xmlDict, null);

//            EvioEvent ev = createSingleEvent(1);
            EvioEvent ev = createSimpleSingleEvent(1);

            for (int i=0; i < eventCount; i++) {
                // Write event to buffer
                writer.writeEvent(ev);
            }

            // All done writing
            writer.close();
        }
        catch (Exception e) {
            e.printStackTrace();
        }

        // Get ready to read
        myBuf.flip();
//        myBuf.position(100);

        return myBuf;
    }


    static void printBytes(ByteBuffer buf) {
        for (int i=buf.position(); i < buf.capacity(); i++) {
            System.out.println("byte = " + buf.get(i));
        }
        System.out.println("\n");
    }

    /** For reading a local file/buffer, take events and put them into
     *  a EvioCompactEventWriter and write a file with it. */
    public static void main5(String args[]) {

        ByteBuffer buf = ByteBuffer.allocate(4);
        buf.order(ByteOrder.BIG_ENDIAN);

        buf.putInt(0, 666);
        printBytes(buf);
        buf.order(ByteOrder.LITTLE_ENDIAN);
        int readVal = buf.getInt(0);
        System.out.println("read value = " + readVal);
        printBytes(buf);

    }


    /** For reading a local file/buffer, take events and put them into
     *  a EvioCompactEventWriter and write a file with it. */
    public static void main(String args[]) {

        int evCount;


        try {
            EvioCompactReader reader = new EvioCompactReader("/daqfs/home/timmer/coda/jevio-4.1/testdata/BSTN2.ev");

            // Get each event in the buffer
            evCount = reader.getEventCount();

            Scanner keyboard = new Scanner(System.in);
            keyboard.next().charAt(0);

            for (int i=0; i < evCount; i++) {
                Thread.sleep(1);
                reader.searchEvent(i+1, 10, 1);
            }

            reader.close();

        }
        catch (EvioException e) {
            e.printStackTrace();
            System.exit(-1);
        }
        catch (Exception e) {
            e.printStackTrace();
            System.exit(-1);
        }
    }




    /** For WRITING a local file. */
    public static void main1(String args[]) {

        // Create buffer with 5 events
        int count = 5, evCount;
        ByteBuffer myBuf = createBuffer(count);
        System.out.println("created buffer, pos = " + myBuf.position() + ", lim = " + myBuf.limit());

        // Create buffer containing bank to add dynamically
        ByteBuffer addBankBuf = createSingleBank(66, 99);
System.out.println("addBankBuf size = " + addBankBuf.remaining());

        List<EvioNode> returnList;

        try {
            EvioCompactReader reader = new EvioCompactReader(myBuf);
            //EvioCompactReader reader = new EvioCompactReader("/tmp/compactTest");

            EvioNode node = reader.getEvent(4);
System.out.println("event 4 pos = " + node.getPosition());

            // Get each event in the buffer
            evCount = reader.getEventCount();

            returnList = reader.searchEvent(4, 7, 7);
            if (returnList.size() < 1) {
                System.out.println("GOT NOTHING IN SEARCH for ev 4, T/N = 7/7");
            }
            else {
                System.out.println("Found " + returnList.size() + " banks for ev 4, T/N = 7/7");
            }

            returnList = reader.searchEvent(2, 7, 7);
            if (returnList.size() < 1) {
                System.out.println("GOT NOTHING IN SEARCH for ev 2, T/N = 7/7");
            }
            else {
                System.out.println("Found " + returnList.size() + " banks for ev 2, T/N = 7/7");
            }

System.out.println("Insert buffer of " + addBankBuf.remaining() + " bytes");
            reader.addStructure(2, addBankBuf);
System.out.println("Insert another buffer of " + addBankBuf.remaining() + " bytes");
            reader.addStructure(2, addBankBuf);

//System.out.println("write buffer to file /tmp/compactTest");
            //reader.toFile("/tmp/compactTest");

System.out.println("event 4 again, pos = " + node.getPosition());

            returnList = reader.searchEvent(4, 7, 7);
            if (returnList.size() < 1) {
                System.out.println("GOT NOTHING IN SEARCH for ev4, T/N = 7/7");
            }
            else {
                System.out.println("Data for T/n = 7/7 ->");
                EvioNode node70 = returnList.get(0);
                ByteBuffer buf = reader.getData(node70);
                DataType dType = node70.getDataTypeObj();
                System.out.println("Get data of type " + dType);
                if (dType == DataType.DOUBLE64) {
                    System.out.println("Double data =");
                    DoubleBuffer dbuf = buf.asDoubleBuffer();
                    for (int k=0; k < dbuf.limit(); k++) {
                        System.out.println(k + "   " + dbuf.get(k));
                    }
                }
            }
        }
        catch (EvioException e) {
            e.printStackTrace();
            System.exit(-1);
        }
    }




    /** For WRITING a local file. */
    public static void main2(String args[]) {

        // Create buffer with 100 events
        int count = 5, evCount = 0, loops = 100000;
        ByteBuffer myBuf = createBuffer(count);
        System.out.println("created buffer, pos = " + myBuf.position() + ", lim = " + myBuf.limit());

        // Create buffer containing bank to add dynamically
        ByteBuffer bankBuf = createSingleBank(66, 99);
        //ByteBuffer bankBuf = createSingleSegment(66);


        try {

            // keep track of time
            long t1, t2, time;
            double rate;

            List<EvioNode> returnList;
            List<BaseStructure> retList1, retList2;

            t1 = System.currentTimeMillis();


            // Doing things the old-fashioned way ...
            if (false) {
                for (int i=0; i < loops; i++) {
                    // Use the original reader which does a deserialization
                    try {
                        // Define a filter to select structures tag=7 & num=0.
                        class myEvioFilter implements IEvioFilter {
                            int tag, num;
                            myEvioFilter(int tag, int num) {this.tag = tag; this.num = num;}
                            public boolean accept(StructureType type, IEvioStructure struct){
                                return ((struct.getHeader().getTag() == tag) &&
                                        (struct.getHeader().getNumber() == num));
                            }
                        };

                        // Create the defined filter
                        myEvioFilter filter70 = new myEvioFilter(7,0);
                        myEvioFilter filter33 = new myEvioFilter(3,3);

                        // Create reader
                        EvioReader reader = new EvioReader(myBuf);

                        // Get each event in the buffer
                        evCount = reader.getEventCount();
                        for (int j=1; j <= evCount; j++) {
                            EvioEvent ev = reader.parseNextEvent();

                            // Use the filter to search "event"
                            retList1 = StructureFinder.getMatchingStructures(ev, filter70);
                            retList2 = StructureFinder.getMatchingStructures(ev, filter33);
                            if (retList1 != null) {
System.out.println("Return list size for search of tag/num = 7/0 -> " + retList1.size());
                               for (BaseStructure bs : retList1) {
                               }
                            }
                        }
                    }
                    catch (IOException e) {
                        e.printStackTrace();
                        System.exit(-1);
                    }

                }

                // calculate the event rate
                t2 = System.currentTimeMillis();
                time = t2 - t1;
                rate = 1000.0 * ((double) loops*evCount) / time;
                System.out.println("rate = " + String.format("%.3g", rate) + " Hz");
                System.out.println("time = " + (time) + " milliseconds");
            }

            t1 = System.currentTimeMillis();


            // Doing things the new way with EvioCompactReader
            if (true) {
                for (int i=0; i < 1; i++) {
                    try {
                        EvioCompactReader reader = new EvioCompactReader(myBuf);
                        //EvioCompactReader reader = new EvioCompactReader("/tmp/compactTest");
                        if (reader.hasDictionary()) {
                            System.out.println(" *** We Have a dictionary ***");
                        }

                        String xmlDict = reader.getDictionaryXML();
                        if (xmlDict != null) {
                            System.out.println("HEY WE GOTTA DICTIONARY String = \n" + xmlDict);
                        }

                        EvioXMLDictionary dictionary = reader.getDictionary();
                        if (dictionary != null) {
                            System.out.println("HEY WE GOTTA DICTIONARY OBJECT !!!");
                        }


                        // Get each event in the buffer
                        evCount = reader.getEventCount();

                        for (int j=0; j < 1; j++) {

//System.out.println("Insert buffer of " + bankBuf.remaining() + " bytes");
                            reader.addStructure(1, bankBuf);

//System.out.println("write buffer to file /tmp/compactTest");
                            //reader.toFile("/tmp/compactTest");

                            returnList = reader.searchEvent(1, 66, 99);
                            //returnList = reader.searchEvent(1, 66, 0);
                            if (returnList.size() < 1) {
                                System.out.println("GOT NOTHING IN SEARCH for T66N99");
                            }
                            else {
                                EvioNode node70 = returnList.get(0);
System.out.println("Data for T66N99 (ret list = " + returnList.size() + ") ->");
                                ByteBuffer buf = reader.getData(node70);
                                DataType dType = node70.getDataTypeObj();
                                System.out.println("Get data of type " + dType);
                                if (dType == DataType.INT32) {
                                    System.out.println("Int data =");
                                    IntBuffer ibuf = buf.asIntBuffer();
                                    for (int k=0; k < ibuf.limit(); k++) {
                                        System.out.println(k + "   " + ibuf.get(k));
                                    }
                                }
                            }

                            //returnList = reader.searchEvent(j+1, "T7N0", dictionary);
                            returnList = reader.searchEvent(3, 7, 0);
                            if (returnList.size() < 1) {
                                System.out.println("GOT NOTHING IN SEARCH for T7N0");
                            }
                            else {
                                System.out.println("Data for T7N0 ->");
                                EvioNode node70 = returnList.get(0);
                                ByteBuffer buf = reader.getData(node70);
                                DataType dType = node70.getDataTypeObj();
                                System.out.println("Get data of type " + dType);
                                if (dType == DataType.DOUBLE64) {
                                    System.out.println("Double data =");
                                    DoubleBuffer dbuf = buf.asDoubleBuffer();
                                    for (int k=0; k < dbuf.limit(); k++) {
                                        System.out.println(k + "   " + dbuf.get(k));
                                    }
                                }
                            }

                            //returnList = reader.searchEvent(j+1,"T3N3", dictionary);
                            returnList = reader.searchEvent(3, 5, 0);
                            if (returnList.size() < 1) {
                                System.out.println("GOT NOTHING IN SEARCH for T5N0");
                            }
                            else {
                                System.out.println("Data for T5N0 ->");
                                EvioNode node33 = returnList.get(0);
                                ByteBuffer buf = reader.getData(node33);
                                DataType dType = node33.getDataTypeObj();
                                System.out.println("Get data of type " + dType);
                                if (dType == DataType.SHORT16) {
                                    System.out.println("Short data =");
                                    ShortBuffer ibuf = buf.asShortBuffer();
                                    for (short k=0; k < ibuf.limit(); k++) {
                                        System.out.println(k + "   " + ibuf.get(k));
                                    }
                                }
                            }


                            returnList = reader.searchEvent(3, 3, 3);
                            if (returnList.size() < 1) {
                                System.out.println("GOT NOTHING IN SEARCH for T3N3");
                            }
                            else {
                                System.out.println("Data for T3N3 ->");
                                EvioNode node33 = returnList.get(0);
                                ByteBuffer buf = reader.getData(node33);
                                DataType dType = node33.getDataTypeObj();
                                System.out.println("Get data of type " + dType);
                                if (dType == DataType.INT32) {
                                    System.out.println("Int data =");
                                    IntBuffer ibuf = buf.asIntBuffer();
                                    for (int k=0; k < ibuf.limit(); k++) {
                                        System.out.println(k + "   " + ibuf.get(k));
                                    }
                                }
                            }

                        }
                    }
                    catch (EvioException e) {
                        e.printStackTrace();
                        System.exit(-1);
                    }
//                    catch (IOException e) {
//                        e.printStackTrace();
//                        System.exit(-1);
//                    }

                }

                // calculate the event rate
                t2 = System.currentTimeMillis();
                time = t2 - t1;
                rate = 1000.0 * ((double) loops*evCount) / time;
                System.out.println("rate = " + String.format("%.3g", rate) + " Hz");
                System.out.println("time = " + (time) + " milliseconds");

            }

        }
        catch (EvioException e) {
            e.printStackTrace();
        }
    }


}
